@@ -23,7 +23,7 @@ from kivy.properties import *
 from kivymd.app import MDApp
 from screeninfo import get_monitors
 
-__version__ = '5.0.5'
+__version__ = '5.2.2'
 
 
 class Content(BoxLayout):
@@ -106,10 +106,60 @@ def measure_time(func):
     return inner
 
 
+@thread
+def file_dialog(callback: callable):
+    from tkinter import Tk, filedialog
+
+    if sys.platform == "darwin":
+        fd = os.popen("""osascript -e 'tell application (path to frontmost application as text)
+set myFile to choose file
+POSIX path of myFile
+end'""")
+        file_path = fd.read()
+        fd.close()
+        file_path = file_path[:-1]
+    else:
+        tk = Tk()
+        tk.withdraw()
+        file_path = filedialog.askopenfilename()
+
+    if file_path:
+        callback(file_path)
+
+
+def get_file(file_path, default=None, create_file_if_not_exist="", is_json=False, encoding='utf-8'):
+    try:
+        with open(file_path, 'r', encoding=encoding) as f:
+            value = f.read()
+
+        if is_json:
+            return json.loads(value)
+        return value
+
+    except FileNotFoundError:
+        if create_file_if_not_exist:
+            set_file(file_path, create_file_if_not_exist)
+
+        return default
+
+    except Exception as e:
+        print(e)
+        return default
+
+
+def set_file(file_path, value, is_json=False, encoding='utf-8'):
+    if is_json:
+        value = json.dumps(value, indent=4)
+
+    with open(file_path, 'w', encoding=encoding) as f:
+        f.write(value)
+
+
 class Kivy4(MDApp):
     dark_mode_icon = StringProperty('')
 
-    def __init__(self, string: str = '', app_name: str = '', dict_of_files: dict = None,
+    def __init__(self, string: str = '', app_name: str = '', dict_of_files: dict[str, str] = None,
+                 list_of_dirs: list[str] = None,
                  screen_size=None, minimum=None, center: bool = True,
                  sun_icon: str = 'white-balance-sunny', moon_icon: str = 'weather-night',
                  main_color: str = 'Blue', icon: str = '', toolbar=False, app_data: bool = True,
@@ -133,6 +183,7 @@ class Kivy4(MDApp):
 
             self.appdata_path = app_data_path
             self.create_files(dict_of_files)
+            self.create_dirs(list_of_dirs)
 
             self.moon_icon = moon_icon
             self.sun_icon = sun_icon
@@ -237,20 +288,34 @@ class Kivy4(MDApp):
         except Exception as e:
             return e
 
-    def set_file(self, file, value, extension='.txt', is_json=False):
+    def create_dirs(self, list_of_dirs):
+        try:
+            if list_of_dirs:
+                for folder in list_of_dirs:
+                    if not os.path.isdir(self.appdata_path + '/' + folder):
+                        os.makedirs(self.appdata_path + '/' + folder)
+
+        except Exception as e:
+            return e
+
+    def set_file(self, file, value, extension='.txt', is_json=False, encoding="utf-8"):
+        if is_json:
+            extension = '.json'
+
         path_to_create = f'{self.appdata_path}/{file}{extension}'
         if is_json:
-            with open(path_to_create, 'w') as f:
-                f.write(json.dumps(value, indent=4))
-        else:
-            with open(path_to_create, 'w') as f:
-                f.write(value)
+            value = json.dumps(value, indent=4)
 
-    def get_file(self, file, default=None, create_file_if_not_exist=False, extension='.txt', is_json=False):
+        with open(path_to_create, 'w', encoding=encoding) as f:
+            f.write(value)
+
+    def get_file(self, file, default=None, create_file_if_not_exist="", extension='.txt', is_json=False, encoding='utf-8'):
+        if is_json:
+            extension = '.json'
         path_of_file = f'{self.appdata_path}/{file}{extension}'
 
         try:
-            with open(path_of_file, 'r') as f:
+            with open(path_of_file, 'r', encoding=encoding) as f:
                 value = f.read()
 
             if is_json:
@@ -259,7 +324,7 @@ class Kivy4(MDApp):
 
         except FileNotFoundError:
             if create_file_if_not_exist:
-                self.set_file(file, default)
+                self.set_file(file, create_file_if_not_exist)
 
             return default
 
@@ -287,12 +352,16 @@ class Kivy4(MDApp):
         except AttributeError:
             return False
 
+    def on_theme_change(self, value):
+        pass
+
     def set_dark_mode(self, value=None, filename='dark mode'):
         if value is None:
             value = darkdetect.theme()
 
         self.set_file(filename, value)
         self.set_dark_mode_icon(value)
+        self.on_theme_change(value)
 
     def reverse_dark_mode(self, filename: str = 'dark mode.txt'):
         try:
@@ -428,8 +497,8 @@ Screen:
         date_dialog.bind(on_save=on_save, on_cancel=self.on_cancel_picker)
         date_dialog.open()
 
-    def show_time_picker(self, on_save, mode='picker'):
-        time_dialog = MDTimePicker(mode=mode)
+    def show_time_picker(self, on_save):
+        time_dialog = MDTimePicker()
         time_dialog.bind(on_save=on_save, on_cancel=self.on_cancel_picker)
         time_dialog.open()
 
@@ -437,31 +506,38 @@ Screen:
         pass
 
     def popup_kivy4(self, title='My popup', content=Content(), cancel_text='CANCEL', okay_text='OKAY',
-                    okay_func=lambda *args: print('yes'), cancel_func=None):
+                    okay_func=lambda *args: print('yes'), cancel_func=None, auto_dismiss=True):
 
         if cancel_func is None:
             cancel_func = lambda *args: self.dismiss()
 
-        if not self.dialog:
-            self.dialog = MDDialog(
-                title=title,
-                type="custom",
-                content_cls=content,
-                buttons=[
-                    MDFlatButton(
-                        text=cancel_text,
-                        theme_text_color="Custom",
-                        text_color=self.theme_cls.primary_color,
-                        on_release=cancel_func
-                    ),
-                    MDFlatButton(
-                        text=okay_text,
-                        theme_text_color="Custom",
-                        text_color=self.theme_cls.primary_color,
-                        on_release=okay_func
-                    ),
-                ],
-            )
+        if self.dialog:
+            self.dismiss()
+
+        buttons = []
+        if cancel_text:
+            buttons.append(MDFlatButton(
+                text=cancel_text,
+                theme_text_color="Custom",
+                text_color=self.theme_cls.primary_color,
+                on_release=cancel_func
+            ))
+
+        if okay_text:
+            buttons.append(MDFlatButton(
+                text=okay_text,
+                theme_text_color="Custom",
+                text_color=self.theme_cls.primary_color,
+                on_release=okay_func
+            ))
+
+        self.dialog = MDDialog(
+            title=title,
+            type="custom",
+            content_cls=content,
+            buttons=buttons,
+            auto_dismiss=auto_dismiss
+        )
 
         self.dialog.open()
 
@@ -470,3 +546,16 @@ Screen:
             self.dialog.dismiss()
         except Exception as e:
             print(e)
+
+    def get_files_content(self, path: str, is_json: bool = False):
+        dir_path = self.appdata_path + '/' + path
+        for file in os.listdir(dir_path):
+            if os.path.isfile(os.path.join(dir_path, file)):
+                with open(os.path.join(dir_path, file), 'r') as f:
+                    if is_json:
+                        yield json.load(f)
+                    else:
+                        yield f.read()
+
+    def delete_file(self, path):
+        os.remove(self.appdata_path + '/' + path)
